@@ -1,6 +1,7 @@
 #possible that lm model returns NAs in coefficients
 #makes diagnostics incalculable
 fastVARX = function(y, x, p, b, getdiag=T) {
+  if(p < 1) stop("p must be a positive integer")
   if(missing(x)) {
     return (fastVAR(y, p, getdiag))
   }
@@ -21,7 +22,7 @@ fastVARX = function(y, x, p, b, getdiag=T) {
   }
 }
 
-.VARXlassocore = function(j, y, x, p, b, Z, y.spec, x.spec) {
+.VARXlassocore = function(j, y, x, p, b, lambda, Z, y.spec, x.spec) {
   colIndex = j[1]
   x.to.remove = which(!x.spec[colIndex,])
   y.to.remove = which(!y.spec[colIndex,])
@@ -46,8 +47,13 @@ fastVARX = function(y, x, p, b, getdiag=T) {
     Z.reduced = Z
   }
        
-  j.lasso = cv.glmnet(Z.reduced, j[-1])
-  j.lasso.s = j.lasso$lambda.min
+  if(is.null(lambda)) {
+    j.lasso = cv.glmnet(Z.reduced, j[-1])
+    j.lasso.s = j.lasso$lambda.min
+  } else {
+    j.lasso = glmnet(Z.reduced, j[-1])
+    j.lasso.s = lambda[colIndex]
+  }
 
   B = rep(0, ncol(Z)+1)  #make vector of coefficients full size
   B.reduced = rep(0, ncol(Z.reduced) + 1)
@@ -63,28 +69,27 @@ fastVARX = function(y, x, p, b, getdiag=T) {
   }
 }
 
-VARXlasso = function(y, x, p, b, y.spec=matrix(1,nrow=ncol(y),ncol=ncol(y)), 
+VARXlasso = function(y, x, p, b, lambda=NULL, 
+  y.spec=matrix(1,nrow=ncol(y),ncol=ncol(y)), 
   x.spec=matrix(1,nrow=ncol(y),ncol=ncol(x)), getdiag=T, numcore=1, ...) {
-  
+ 
+  if(p < 1) stop("p must be a positive integer")
+  if(!is.matrix(y)) {
+    stop("y must be a matrix (not a data frame).  Consider using as.matrix(y)")
+  }
   varz = VARXZ(y,x,p,b)
   Z = varz$Z
   y.augmented = rbind(1:ncol(y),varz$y.p)
   
-  if(numcore==1) {
+  if(numcore==1 | !require(multicore)) {
     var.lasso = apply(y.augmented, 2, .VARXlassocore, y=y,x=x,p=p,b=b,
-      Z=Z, y.spec=y.spec, x.spec=x.spec)
+      lambda=lambda, Z=Z, y.spec=y.spec, x.spec=x.spec)
   } else {
-    if(!require(multicore)) {
-      #Cannot load multicore, use apply instead
-      var.lasso = apply(y.augmented, 2, .VARXlassocore, y=y,x=x,p=p,b=b,
-        Z=Z, y.spec=y.spec, x.spec=x.spec)
-    } else {
-      y.augmented.list = c(unname(as.data.frame(y.augmented)))
-      var.lasso = mclapply(y.augmented.list, .VARXlassocore, y=y,x=x,p=p,b=b,
-        Z=Z, y.spec=y.spec, x.spec=x.spec, mc.cores=numcore, ...)
-      var.lasso = matrix(unlist(var.lasso), nrow=(varz$k+1), ncol=ncol(y))
-      colnames(var.lasso) = colnames(y)
-    }
+    y.augmented.list = c(unname(as.data.frame(y.augmented)))
+    var.lasso = mclapply(y.augmented.list, .VARXlassocore, y=y,x=x,p=p,b=b,
+      lambda=lambda, Z=Z, y.spec=y.spec, x.spec=x.spec, mc.cores=numcore, ...)
+    var.lasso = matrix(unlist(var.lasso), nrow=(varz$k+1), ncol=ncol(y))
+    colnames(var.lasso) = colnames(y)
   }
   rownames(var.lasso) = c('intercept', colnames(Z))
   if(getdiag) {
@@ -99,13 +104,17 @@ VARXlasso = function(y, x, p, b, y.spec=matrix(1,nrow=ncol(y),ncol=ncol(y)),
 }
 
 VARXpredict = function(y, x, p, b, B, xnew, n.ahead=1, threshold) {
+  if(p < 1) stop("p must be a positive integer")
   y.pred = matrix(nrow=n.ahead, ncol=ncol(y))
   colnames(y.pred) = colnames(y)
   for(i in 1:n.ahead) {
-    Z.ahead = c(1,
-      as.vector(t(y[((nrow(y)):(nrow(y)-p+1)),])),
-      as.vector(t(x[((nrow(x)):(nrow(x)-b+1)),]))
-    )
+    Z.ahead.y = as.vector(t(y[((nrow(y)):(nrow(y)-p+1)),]))
+    if(b == 0) {
+      Z.ahead.x = xnew[1,]
+    } else {
+      Z.ahead.x = as.vector(t(x[((nrow(x)):(nrow(x)-b+1)),]))
+    }
+    Z.ahead = c(1, Z.ahead.y, Z.ahead.x)
     y.ahead = Z.ahead %*% B
     if(!missing(threshold)) {
       threshold.indices = which(y.ahead < threshold)
@@ -123,6 +132,7 @@ VARXpredict = function(y, x, p, b, B, xnew, n.ahead=1, threshold) {
 }
 
 VARXZ = function(y, x, p, b) {
+  if(p < 1) stop("p must be a positive integer")
   if(is.null(colnames(y))) {
     colnames(y) = sapply(1:ncol(y), function(j) {
       paste('y',j,sep='')
@@ -151,15 +161,19 @@ VARXZ = function(y, x, p, b) {
     endIndex = i*ny
     paste(Z.p.names[startIndex:endIndex], '.l', i, sep='')
   })
-  Z.b = do.call('cbind', (sapply(0:(b-1), function(k) {
-    x[(p.max-k):(T-1-k),]
-  }, simplify=F)))
-  Z.b.names = colnames(Z.b)
-  colnames(Z.b) = sapply(1:b, function(i) {
-    startIndex = (i-1)*nx + 1
-    endIndex = i*nx
-    paste(Z.b.names[startIndex:endIndex], '.l', i, sep='')
-  })
+  if(b == 0) {
+    Z.b = x[(1+p):T,]
+  } else {
+    Z.b = do.call('cbind', (sapply(0:(b-1), function(k) {
+      x[(p.max-k):(T-1-k),]
+    }, simplify=F)))
+    Z.b.names = colnames(Z.b)
+    colnames(Z.b) = sapply(1:b, function(i) {
+      startIndex = (i-1)*nx + 1
+      endIndex = i*nx
+      paste(Z.b.names[startIndex:endIndex], '.l', i, sep='')
+    })
+  }
   Z = cbind(Z.p, Z.b)
 
   return ( list (
